@@ -1,5 +1,13 @@
 ## Execute on attach
 
+## available packages, to check whether RODBC and RPostgreSQL are
+## already installed
+.get.installed.pkgs <- function () {
+    tolower(attr(installed.packages(), "dimnames")[[1]])
+}
+
+## ------------------------------------------------------------------------
+
 ## define local variables
 
 .onAttach <- function(libname, pkgname)
@@ -21,4 +29,6 @@
     .localVars$conn.type <- list()
     for (i in seq(along=.supported.connections))
         .localVars$conn.type[[tolower(.supported.connections[i])]] <- integer(0)
+
+    packageStartupMessage("\nTo launch the graphical user interface, run the function pivotalr() !")
 }
