@@ -9,8 +9,7 @@
     ## Only newer versions of MADlib are supported
     idx <- .localVars$conn.id[.localVars$conn.id[,1] == conn.id(data), 2]
     if (identical(.localVars$db[[idx]]$madlib.v, numeric(0)) ||
-        .madlib.version.number(conn.id(data)) < 0.6 ||
-        .madlib.version.number(conn.id(data)) > 0.7)
+        .madlib.version.number(conn.id(data)) < 0.6)
         stop("MADlib error: Please use Madlib version 0.6 or 0.7!")
 }
 
@@ -46,18 +45,70 @@
 ## ------------------------------------------------------------------------
 
 ## get the result
-.get.res <- function (sql, tbl.output, conn.id)
+.get.res <- function (sql, tbl.output = NULL, conn.id)
 {
     ## execute the linear regression
     res <- try(.db.getQuery(sql, conn.id), silent = TRUE)
     if (is(res, .err.class))
-        stop("Could not run MADlib linear regression !")
+        stop("Could not run SQL query !")
 
     ## retreive result
-    res <- try(.db.getQuery(paste("select * from", tbl.output), conn.id),
-               silent = TRUE)
-    if (is(res, .err.class))
-        stop("Could not retreive MADlib linear regression result !")
+    if (!is.null(tbl.output)) {
+        res <- try(.db.getQuery(paste("select * from", tbl.output),
+                                conn.id),
+                   silent = TRUE)
+        if (is(res, .err.class))
+            stop("Could not retreive result from SQL query !")
+    }
 
     res
 }
+
+## ------------------------------------------------------------------------
+
+.get.groups <- function (x)
+{
+    if (length(x$grp.cols) != 0) {
+        res <- list()
+        for (col in x$grp.cols)
+            res[[col]] <- x[[col]]
+        return (res)
+    } else {
+        return (NULL)
+    }
+}
+
+.get.groups.grps <- function (x)
+{
+    n <- length(x)
+    for (i in seq_len(n)) {
+        if (i == 1)
+            res <- .get.groups(x[[1]])
+        else {
+            tmp <- .get.groups(x[[i]])
+            for (col in x[[i]]$grp.cols)
+                res[[col]] <- c(res[[col]], tmp[[col]])
+        }
+    }
+    res
+}
+
+groups.lm.madlib <- function (x)
+{
+    .get.groups(x)
+}
+
+groups.logregr.madlib <- function (x)
+{
+    .get.groups(x)
+}
+
+groups.lm.madlib.grps <- function (x)
+{
+    .get.groups.grps(x)
+}
+
+groups.logregr.madlib.grps <- function (x)
+{
+    .get.groups.grps(x)
+}
